/**
 * Created by Administrator on 2016/1/19.
 */
var express = require('express');
var router = express.Router();
var baseRoute = require('./base.js');
//var stateService = require('../service/state.js');
var pageViewService = require('../service/pageview.js');
var dateutil = require('../modules/dateutil.js');
var pageViewAllService = require('../service/pageviewall.js');
var pageViewDetailService = require('./../service/pageviewdetail.js');

router.get('/', function (req, res, next) {
    res.render('result', {title: '统计结果'});
});

/*
 * 查询所有数据
 */
router.get('/pv', function (req, res, next) {
    pageViewService.find(baseRoute.success.bind(res), baseRoute.fail.bind(res));
});
/**
 * 查询某一天的所有数据
 * 共144个时间段 24*6
 */
router.get('/pv/day', function (req, res, next) {
    var d = req.query;
    var spance = d.spance;
    spance = spance ? parseInt(spance) : 1;
    var startAndEndTime = dateutil.getStartAndEndInDay(d.date);
    var startTime = startAndEndTime.startTime;
    var endTime = startAndEndTime.endTime;
    var offset = 5 * 1000; //偏移
    var spanceMinu = 10 * spance; //间隔分钟

    var SNMap = {}; //每个时间段的数据总量 count, uvCount, ipCount, tokenCount
    pageViewService.find({
        urlpath: d.urlpath, //精确
        startTime: {$gt: new Date(startTime.valueOf() - offset)},
        endTime: {$lt: new Date(endTime.valueOf() + offset)}
    }, function (list) {
        list.forEach(function (o) {
            var sn = _getSN(o.startTime, o.endTime);
            _setSNMap(sn, o);
        });
        _render();
    });

    /**
     * 获得时间点对应的序号
     * 00:05:00 -> 0
     * 00:10:00 -> 1
     * @param startTime
     * @param endTime
     * @returns {number}
     * @private
     */
    function _getSN(startTime, endTime) {
        var beTime = new Date((startTime.valueOf() + endTime.valueOf()) / 2);
        var minu = beTime.getHours() * 60 + beTime.getMinutes();
        var sn = Math.floor(minu / spanceMinu);
        return sn;
    }

    function _setSNMap(key, o) {
        if (SNMap[key]) {
            _setValue('count'),
                _setValue('uvCount'),
                _setValue('ipCount'),
                _setValue('tokenCount');
        } else {
            SNMap[key] = {
                count: o.count,
                uvCount: o.uvCount,
                ipCount: o.ipCount,
                tokenCount: o.tokenCount
            };
        }
        function _setValue(k) {
            SNMap[key][k] += o[k];
        }
    }

    function _render() {
        var arr = [];
        var spaceTotal = 24 * 60 / spanceMinu;
        var total = 0;
        var totalMap = {
            count: 0,
            uvCount: 0,
            ipCount: 0,
            tokenCount: 0,
            sn: '',
            timePoint: '全部'
        };
        for (var i = 0; i < spaceTotal; i++) {
            var t = SNMap[i] ? SNMap[i] : {count: 0, uvCount: 0, ipCount: 0, tokenCount: 0};
            arr.push({
                sn: i + 1,
                timePoint: _getTimePoint(i),
                count: t.count,
                uvCount: t.uvCount,
                ipCount: t.ipCount,
                tokenCount: t.tokenCount
            });
            totalMap.count += t.count,
                totalMap.uvCount += t.uvCount,
                totalMap.ipCount += t.ipCount,
                totalMap.tokenCount += t.tokenCount;
        }
        arr.push(totalMap);
        baseRoute.success.call(res, arr);

        //获得序号对应的时间点值
        function _getTimePoint(num) {
            var minu = num * spanceMinu;
            var h = Math.floor(minu / 60);
            var m = minu % 60;
            return _getStrNum(h) + ':' + _getStrNum(m);
        }

        function _getStrNum(i) {
            if ((i + '').length === 1) {
                return '0' + i;
            } else {
                return '' + i;
            }
        }
    }
});

//根据模式全局查询
router.get('/pv/cond', function (req, res, next) {
    var q = req.query.q;
    var reg = new RegExp(q);
    pageViewService.find({
        urlpath: reg,
    }, function (list) {
        var totalMap = {
            count: 0,
            uvCount: 0,
            ipCount: 0,
            tokenCount: 0,
            sn: '',
            timePoint: '全部'
        };
        var currTime = new Date();
        var startTime = currTime,
            endTime = currTime,
            createTime = currTime;
        if (list && list.length) {
            startTime = list[0].startTime,
                endTime = list[0].endTime;
            list.forEach(function (o) {
                totalMap.count += o.count,
                    totalMap.uvCount += o.uvCount,
                    totalMap.ipCount += o.ipCount,
                    totalMap.tokenCount += o.tokenCount;
                if (+o.startTime < +startTime) {
                    startTime = o.startTime;
                }
                if (+o.endTime > +endTime) {
                    endTime = o.endTime;
                }
            });
        }
        baseRoute.success.call(res, [totalMap]);
    });
});

//按天查询数据概况
router.get('/pv/dayall', function (req, res, next) {
    var d = req.query;
    //pageViewAllService.find({urlpath: d.urlpath}, baseRoute.success.bind(res), baseRoute.fail.bind(res));
    pageViewAllService.model.
    find({urlpath: d.urlpath}).
    sort({startTime: -1}).exec(function (err, list) {
        var result = [];
        list.forEach(function (o, i) {
            var t = {
                sn: i + 1,
                timePoint: dateutil.convertTimeToStr(o.startTime).substr(0, 10),
                count: o.count,
                uvCount: o.uvCount,
                tokenCount: o.tokenCount,
                ipCount: o.ipCount
            }
            result.push(t);
        });
        baseRoute.success.call(res, result);
    });
});

//按天查询页面详情
router.get('/pv/pagedetailbyday', function (req, res, next) {
    var d = req.query;
    var date = d.date;
    var startAndEndDate = dateutil.getStartAndEndInDay(date);
    pageViewDetailService.model.
    find({startTime: startAndEndDate.startTime}).sort({startTime: -1, count: -1}).exec(function(err, list){
        if(err) console.error(err);
        var result = [];
        list.forEach(function (o, i) {
            var t = {
                sn: i + 1,
                timePoint: o.urlpath,
                count: o.count,
                uvCount: o.uvCount,
                tokenCount: o.tokenCount,
                ipCount: o.ipCount
            }
            result.push(t);
        });
        baseRoute.success.call(res, result);
    });
});

module.exports = router;